#ifndef AVAILABLEMAP_H__
#define AVAILABLEMAP_H__

#include <vector>
#include <bitset>
#include <string>
#include <iostream>

/** 
 *            Selfing History
 *
 *   The selfing history of a node is represented by an integer.  For various
 * reasons the cnversion from the integer from the selfing history to the integer
 * is "the wrong way".  So a selfing history (going back in time) of
 * Self -> OutCross -> Self -> Self can be represented by SOSS and this
 * is:
 *
 * 0 1 0 0 = 8
 *
 * The Time is the time at which the joining happens 
 * Not the level at which the nodes are Available
 * 
 * which is the position in the available array (0 offset) 
*/


/** Need another class to map from the 
 *  available history to a vector of selfing and non-selfing  */

typedef std::bitset<16> SelfingHistory;


std::bitset<16> outcrosses(int selfH, int gen);
std::string printoutcrosses(int selfH, int gen);

class mpforest;

class availInfo {
public:
  
  int generation() const {
    return generation_;
  }
  void generation(int gen) {
    generation_=gen;
  }
  void reset() {
    SH=0;
    which_=0;
    generation_=0;
  }
  int which() const {
    return which_;
  }
  void which(int wh) {
    which_=wh;
  }
  int SelfHist() const {
    return (int) SH.to_ulong();
  }
  void SelfHist(int sh) {
    SH=sh;
  }

  void SelfHist(const std::bitset<16> &outcrosses) {
    //   std::cerr << "oc: " << outcrosses << std::endl;
    for (int i=0;i<generation();i++) {
      //     assert(generation()-1-i>=0 && generation()-1-i <16);
      SH[i] = outcrosses[generation()-1-i];
    }
    //   std::cerr << "SH: "<< SH << std::endl;
  }
  bool isroot() const {
    return which_<0;
  }
  bool selfing(int time,int gen) const {
    return !SH[gen-1-time];
  }
  bool outcross(int time,int gen) const {
    return SH[gen-1-time];
  }
  void print(std::ostream &o, const char *lab="") const {
    o << lab << "  Generation " << generation() 
      << " which = " << which() << " " << "(" << SH <<") " 
      << " with selfing history " << SH.to_ulong() << ":   "        ;
    for (int i=0;i<generation()-1;i++) {
      o << (outcross(i,generation())?"O":"S") << "->";
    }
    o << (outcross(generation()-1,generation())?"O":"S") << std::endl;
  };
  bool sensible(const mpforest &mp) const; 
private:
  int generation_;
  int which_;
  /** The backwards in time selfing history 
   * i.e. SH[0] is the history at time 0, SH[1] is the history
   * at time 1 (in the past) etc                               */
  std::bitset<16> SH;
  /** Function to convert from a backwards time to the probability ordering */
 //  void setSelfHist() {
//     //    std::cerr << "setting selfHist\n";
//     std::bitset<16> a(selfHist);
//     for (int i=0;i<generation_;i++) a[i]=SH[generation_-1-i];
//     for (int i=generation_;i<16;i++) a[i]=0;
//     selfHist = int(a.to_ulong());
//   }
//   void setBitSet() {
//     std::bitset<16> a(selfHist);
//     for (int i=0;i<generation_;i++) SH[i]=a[generation_-1-i];
//   }
};

/** A class to map available nodes and the vector of outcrossing selfing 
 *  events onto a probability vector 
 */
class availableMap {
public:
  availableMap(const mpforest &sf);
  // get a position from information
  int pos(const availInfo &AI);
  // get information from a position
  availInfo get(int pos);

  int length() const {
    return cumsum.back();
  }
    
  int startpos(int generation) const {
    return cumsum[generation];
  }
private:
  std::vector<int> cumsum;

};




#endif
