#ifndef CHAIN_H__
#define CHAIN_H__

#include "mpforest.H"
#include "popFreq.H"
#include "gsl_rand.H"
#include "gsl_distributions.H"
#include "genotype.H"


template <typename T>
T reflect01(T val) {
  for (;;) {
    if (val<0.0) val=-val;
    else if (val>1.0) val=2.0-val;
    else return val;
  }
}
template <typename T>
T reflect0(T val) {
    if (val<0.0) val=-val;
    return val;
}


template <typename T>
T reflect(double mn,T val) {
    if (val<mn) val=2.*mn-val;
    return val;
}

/** A class to encapsulate the current state of a chain and the methods
 * for stepping through the chain.  It should be easy to inherit from this
 * and add a heating parameter                                              */

class chain {

public:
  /** Constructors and destructors                     */
  chain(const std::vector<genotype> &g,const popFreq &ppf,
	ctsdistribution &sp, ctsdistribution &Np
	, int T ,rng &rnd,double tuneN=20
	,double tunes=0.2,double startN=0,double starts=0.0);
  ~chain();
  
  /** The log of the posterior.  If repeel then the trees are repeeled 
   * (as should be done if the selfing rate is changed                     */ 
  double lposterior(bool repeel=true) {
    return  x->peel(s,pf,repeel) +  x->lprobtrees(s,N,T)
      +NPrior.log_pdf(N)+ sPrior.log_pdf(s);
  }
  /** The number of roots in the trees                                     */
  int nroots() const {
    return x->nroots();
  }
  /** parition the descendents by their matrilineal roots                  */
  std::vector<int> rootpartition() {
    return x->partition(-1);
  }
  /** Partition the sam,ple by their most recent outcrossing ancestors     */
  std::vector<int> MROApartition() {
    return x->partitionMROA();
  }
  /** Gibbs sample by removing and adding a single sample     
   * Note that this also changes the order of the sample                   */            
  void gibbs() {
    x->gibbs_sample( T,  N,  s, pf, r );    
  }
  void gibbs_reorder() {
    x->reorder(r);
  }
  bool metro_addremovek(int k) {
    assert(k>0&&k<int(x->nsamples()));
    return  x->metro_addremove_k(k,T,N,s,pf,r);
  }  

  /** metropopolis functions for changinh s and N                         */
  bool metro_s();
  bool metro_N(bool logarithm=false);
  bool metro_sN();
  bool metro_pf();
private:
  const std::vector<genotype> &gg;
  mpforest *x;
  std::vector<int> perm;
public:
  double s,N;
  int T;
  popFreq pf;
  rng &r;
  ctsdistribution &sPrior;
  ctsdistribution &NPrior;
  double tune_N,tune_s;
  /**
   * metropolis step on the population frequencies
   */
  
};

#endif
