#ifndef NEWIO_H___
#define NEWIO_H___

#include <cstdio> // for EOF

#include <vector>
#include <string>
#include <iostream>
#include <iterator>
#include <string>
#include <fstream>
#include <iomanip>
#include <sstream>

#ifdef USETNT
#include <tnt/tnt.h>
#endif

template <typename T> void fromstring(const std::string &s, std::vector<T> &a, const char sep=',');
template <class T> std::vector<T> readvector(std::istream &in,T dummy);
template <class T> void printvectorb(std::ostream &o,const std::vector<T> &u,int len=5,int width=14);

template <typename T>
const std::string stemLeafFilename(const T &stem, const char *leaf,  char sep='.');

/** A class to make logging to file of errors and messages easier    */
class logger {
public:
  logger(const char *filename, bool usestdcerr=true)
    :logfile(filename),usecerr(usestdcerr) {}
  ~logger() {
    logfile.close();
  }
   void operator()(const std::exception &e) throw() {
     logfile << "error: " << e.what();
     if (usecerr) std::cerr << "error: " << e.what();
     logfile.flush();
  }
  void operator()(const char *message) {
    logfile << message;
    if (usecerr) std::cerr << message;
    logfile.flush();
  }
private:
  std::ofstream logfile;
  bool usecerr;
};


/** A class to produce a text progress bar - only useful if length > 20   */
class progressBar {
public:
  progressBar(std::ostream &o,const char *comm, int length):oo(o),len(length),finished(false),curr(-1){
    start(comm);
  };
  ~progressBar() {
    if (!finished) finish();
  }
  void start(const char *comm) {
    oo << comm << " [" <<  std::flush;
  }
  void restart(const char *comm,int length) {
    if (!finished) finish();
    len = length;
    curr=-1;
    start(comm);
  }
  void update(int where) {
    int pr=int(20.0*where/(double)len);
    if (pr>curr) {
      curr=pr;
      if (!(pr%4)) 
	oo << 5*pr << "%" << std::flush;
      else 
	oo << "." << std::flush;
    }
  }
  void finish() {
    if (curr<20) oo << "100%";
    oo << "]" << std::endl;
    finished=true;
  }
private:
  std::ostream &oo;
  int len;
  bool finished;
  int curr;
};
/** A class to produce a text progress bar - most useful if length < 40   */
class simpleBar {
public:
  simpleBar(std::ostream &o,const char *comm):oo(o),finished(false){
    start(comm);
  };
  ~simpleBar() {
    if (!finished) finish();
  }
  void start(const char *comm) {
    oo << comm << " [" <<  std::flush;
  }
  void restart(const char *comm) {
    if (!finished) finish();
    start(comm);
  }
  void update() {
    oo << "." << std::flush;
  }
  void finish() {
    oo << "]" << std::endl;
    finished=true;
  }
private:
  std::ostream &oo;
  bool finished;
};

/** My own expception class - need to work on this           */
class ioerror: public std::exception {
 public:
  ioerror(const char *mess)
    :message(mess)  {
    std::cerr << message;
  };
  ~ioerror() throw() {
  }
  
  const char *what() const throw()  {
     return message.c_str();
  }
private:
   const std::string message;
};


/**  Read a list of comma separated Ts from a string   
     cannot return a because otherwise template would not work*/
template <typename T>
void fromstring(const std::string &s, std::vector<T> &a, const char sep) 
{
  std::istringstream iss(s);
  a.resize(0);

  T itmp;
  char ch;
  for (;;) {
   
    iss >> itmp; 
    if (!iss) break;
    a.push_back(itmp);
    iss >> ch;
    
    //  std::cout << itmp << ch << std::endl;    
    if (ch != sep) {
      std::ostringstream oss;
      oss << "problem in fromstring,\nExpected separator " << sep << 
	" and got " << ch << std::endl;
      throw ioerror(oss.str().c_str());
    }
   
  }
 
}  
template <typename T>
const std::string stemLeafFilename(const T &stem, const char *leaf,  char sep='.')
{
  std::ostringstream oss;
  oss << stem << sep << leaf;
  return oss.str().c_str();
}

typedef enum volum {quiet, loud} volume;

std::vector <std::string> readstringline(std::istream &in
			       ,int suggestedlength=10, char until='\n');
std::vector<std::vector<std::string> > readstringlines(std::istream &in);
std::vector <int> readintvector(std::istream &in,bool usechar=false, char until=EOF);
std::vector <std::vector<int> > readcharintmat(std::istream &in,size_t &cols, bool usechar=false);

int skipblank(std::istream &in, char until='\n');
void checkreadcharacter(std::istream &in ,const char wh, const char *message="error");
int skipline(std::istream &in);
void Tokenize(const std::string& str,std::vector<std::string>& tokens
	      ,const std::string& delimiters = " ");
bool skipto(std::istream &in, const char *pattern, const char comment='#');
bool findparametername(std::istream &in, const char *pattern);

std::vector<double> stringtodouble(std::vector<std::string> s,size_t from=0);
template <class T>
void printvector(std::ostream &o,std::vector<T> &u,bool newline=false,const char *between=" ");
template <class T>
void printvector(std::ostream &o,const std::vector<T> &u,bool newline=false,const char *between=" ");
template <class T>
void printvectorb(std::ostream &o,const std::vector<T> &u,int len=5,int width=14);
/*******************
 *
 *
 *
 *******************/
/** Little function to output an STL   */
template <class T>
void printvector(std::ostream &o,std::vector<T> &u,bool newline,const char *between) {
  std::copy(u.begin(),u.end(),std::ostream_iterator<T>(o,between));
  if (newline) o << std::endl;
}
template <class T>
void printvector(std::ostream &o,const std::vector<T> &u,bool newline,const char *between) {
  std::copy(u.begin(),u.end(),std::ostream_iterator<T>(o,between));
  if (newline) o << std::endl;
}
template <class T>
void printvectorb(std::ostream &o,const std::vector<T> &u,int len,int width) {

  for (size_t i=0;i<u.size();i++) {
    if (i%len==0) o << std::endl << std::setw(5) << i << ": ";
    o << std::setw(width) << u[i] << " ";
  }
  o << std::endl;
}
#ifdef USETNT
/** read a vector from the TNT::Array format    */
template <class T>
std::vector<T> readvector(std::istream &in,T dummy) {
  TNT::Array1D<T> a;
  in >> a;

  return std::vector<T>(&a[0],&a[0]+a.dim());
}
#endif
template <typename T>
std::vector <std::vector<T> > readmat(std::istream &in,int rows, int cols, T dummy)
{
  std::vector<std::vector<T> > a(rows,std::vector<T>(cols));
  for (int i=0;i<rows;i++) {
    for (int j=0;j<cols;j++) in >> a[i][j];
  }
  return a;
}
/** returns a pointer and a length to a string representation 
 * (provided that T  has a << operator                         */
template <typename T>
std::string tostring(T *start, int length) {
  std::ostringstream oss;
  std::copy(start,start+length,std::ostream_iterator<T>(oss,""));
  return oss.str();
}

double getval(const std::string &str, const char *val,bool loud=false);
std::string getstringfromkeyboard(const char*message, const char *def_val);


int findstart(std::istream &in,char *pattern);
int skipspace(std::istream &in);
char *readfromquotes(std::istream &in,int *len);

/***************************************************************************/
/*    Now for scanning of old style parameter files                        */
/***************************************************************************/
template <typename T>
T scan(std::istream &in, const char *namestring,const T &default_val, volume vol=quiet)
{
    if (findparametername(in,namestring)) {
      T val;
      in >> val;
      return val;
    } else {
        if (vol==loud) {
        std::cerr << namestring
		  << " not found in parameter file, using "
		  << default_val << std::endl;
        }
      return default_val;
    }
}
//
//
//
template<typename T>
std::vector <T> readvec(std::istream &in, char until, T dummy) 
{

    std::vector <T> data;
    T x;
    for (;;) {
      int ch;
      do {
	ch = in.get();
      } while (ch!=until&&isspace(ch));

      if ((isdigit(ch))||(ch=='.')||(ch=='-')) {
	in.putback(char(ch));
	in >> x;

	data.push_back(x);
      } else  break;
    }
    return data;
}


std::vector<int> intvector_scan(std::istream &in, const char *namestring);
char *namescan(std::istream &in, char *namestring, char *default_val);
int *charintvector_scan(std::istream &in, char *namestring,int *default_val,volume vol=quiet);
int nextname(std::istream &in, char *filename);
int get_doublesfrombrackets(std::istream &in,double *par);
bool findparametername(std::istream &in, const char *pattern);
#endif
