// Time-stamp: <2006-10-24 11:44:00 uijw>

#ifndef POPFREQ_H__
#define POPFREQ_H__

#include <iostream>
#include <vector>
#include "gsl_rand.H"

class genotype;

/** A class to keep track of population allele frequencies         */
class popFreq {
public:
  /** constructors and destructor                                  */
  popFreq():nloci_(0) {}
  popFreq(const std::vector<int> &nalleles,double a,rng &r);
  popFreq(const std::vector<genotype> &d,bool addone=true);
  popFreq(const std::vector<genotype> &d, double a,rng &r);
  popFreq(std::istream &in);
  popFreq(int loci, double theta,rng &r);
  ~popFreq() {}

  std::vector<char> haplotype(rng &r) const ;
  genotype sample(rng &r) const;

  double &operator()(int locus,int all )  {
    assert(locus<nloci_&&all<alleles_[locus]);
    return p_[locus][all];
  }
  double operator()(int locus,int all )  const{
    assert(locus<nloci_&&all<alleles_[locus]);
    return p_[locus][all];
  }
  const std::vector<double> &operator[](int locus) const  {
    return p_[locus];
  }

  int nloci() const {
    return nloci_;
  }
  int alleles(int locus) const {
    assert(locus<nloci_);
    return alleles_[locus];
  }
  const std::vector<int> &alleles() const {
    return alleles_;
  }

  double psamplehom(int locus, int allele) const; // homozygote
  // homozygote, but can be just double (not allele1).
  double psamplehom(int locus, int allele1, int allele2) const;
  double psample(int locus, int allele1, int allele2) const;

  double lpsample(const genotype &als) const;
  double lpsample(int locus, const std::pair<int,int> &als) const ;
 //  void addsample(const std::vector<std::pair<int,int> > &als) const {
//     // nothing to do for this model
//   }

private:
 int nloci_;
 std::vector<std::vector<double> >  p_;
 std::vector<int> alleles_;
};

std::ostream &operator<<(std::ostream &o, const popFreq &pf);




#endif

