/** @file   
 * Some very simple utility functions (mainly for mathematics)   */
#ifndef CPPUTIL_H_
#define CPPUTIL_H_
#include <cstdlib>
#include <cmath>
#include <iostream>
#include <vector>



using std::cerr;

enum sex{male,female};

#ifdef WIN32
/* const double  M_PI = 3.14159265358979323846; */
/* const double  M_LN2 =0.69314718055994530942; */
/* double lgamma(double xx); */
#endif

// some mathematical constants
#ifndef USE_R
const double  M_1_SQRT_2PI=	0.398942280401432677939946059934;
const double  M_SQRT_32=	5.656854249492380195206754896838;
#endif
const double  LMY_SQRT_2PI =-0.918938533204672669540968854562;


/// A very simple error class
class error {
public:
/** Only consists of a constructor that prints a message and then exits */
  error(const char *text) {
    cerr << text;
    exit(EXIT_FAILURE);
  }
};
/// A very simple warning class
class warning {
public:
/** Only consists of a constructor that prints a message */
    warning (const char *text) {
	std::cerr << text << std::endl;
    }
};

/// class that simulates a factorial function- but includes a lookup table.
class lfactorial {
public:
  lfactorial() {
    for (int i=0;i<32;i++) a[i]=0.0;
  }
  double operator()(int n) {
    if (n<=1) return 0.0;
    if (n<=31) return (a[n]>0.0) ? a[n] : (a[n]=lgamma((double)(n+1)));	
    else return lgamma((double)(n+1));
  };
 private:
  double a[32];
};

int pow2(int x);
/** returns the sum of the powers of 2 up to n
    sumpow2(6) = 2+4+8+16+32+64 */
int sumpow2(int x);


// functions
double max(double x, double y);
double max(double *x, int n);
// sorting
void hpsort(unsigned long n, double  ra[]);
void hpsort(unsigned long n, int ra[]);
// indexing
void indexi(unsigned long n, int arr[], int indx[]);
// quantile calculation
void quantiles(double *res,double *x, int nx, double *p,int np);
// mathematical functions
double log_D(const double *b, int n);

// functionoids
extern lfactorial lfactrl;



/** is an integer the square of a natural number */
bool IsSquare(int n, int &siz);

#endif
