#ifndef UTILITYFUNCTIONALS_H_
#define UTILITYFUNCTIONALS_H_

/* template <typename T> */
/* class cumsum: public unary_function<T,T> { */
/* public: */
/*     cumsum():sum(0){} */
/*     T& operator()(const T &item) { */
/* 	sum += T; */
/* 	return sum; */
/*     } */
/* private: */
/*     T sum; */
/* }; */
#include <vector>
#include <algorithm>
#include <numeric>
#include <set>
#include <cmath>
#include <iostream>
#include <iterator>  // for ostream_iterator

template <typename ITOR> std::vector<int> tabulate(ITOR first, ITOR last);
template <typename T> std::ostream &print(std::ostream &o
		    ,const std::vector<std::vector<T> > &a);

template <typename T> T rescale(  T *a, int count);
template <typename ITOR> std::vector<std::vector<int> >tabulate(ITOR first, ITOR last, ITOR f2, ITOR l2);
template <typename T> void keep_first_copy(std::vector<T> &a);








template <typename T>
class cumsum {
public:
    cumsum():sum(0){}
    T& operator()(const T &item) {
	sum += item;
	return sum;
    }
private:
    T sum;
};
//// from Meyers, Effective STL p38
struct DeleteObject {
  template <typename T> 
  void operator()( const T* ptr) const 
  {
    delete ptr;
  }
};

/** A class for set comparators   */
//template<class T>
//struct less {
//  bool operator()(T x, T y) { return x<y; }
//};
template<class T>
struct moreorequal {
  bool operator()(T x, T y) { return !(x<y); }
};


template <typename T>
T rescale(  T *a, int count) 
{
  T sm = std::accumulate(a,a+count,0.0);
  sm /= (double)count;
  for (int i=0;i<count;i++) a[i]/=sm;
  return log(sm);
}

// only written for character variables
template <typename ITOR>
std::vector<int> tabulate(ITOR first, ITOR last)
{
  std::vector<int> b(*(std::max_element(first,last))+1);
  ITOR i=first;
  while (i!=last) {
     b[*i]+=1;
     i++;
  }
  return b;
}

// only written for ints
template <typename ITOR>
std::vector<std::vector<int> >tabulate(ITOR first, ITOR last, ITOR f2, ITOR l2)
{
  int cols=*(std::max_element(f2,l2));
  std::vector<std::vector<int> > 
    b(*(std::max_element(first,last))+1,cols);

  ITOR i=first;
  ITOR j=f2;
  while (i!=last) {
     b[*i][*j]+=1;
     i++;j++;
  }
  return b;
}

template <typename T>
std::ostream &print(std::ostream &o
		    ,const std::vector<std::vector<T> > &a)
{
  for (int i=0;i<a.size();i++) { 
    std::copy(a[i].begin(),a[i].end()
	      ,std::ostream_iterator<T >(o," "));
    o << std::endl;
  }
  return o;
}


/// class that simulates a factorial function- but includes a lookup table.
class lfactorl {
public:
  lfactorl() {
    for (int i=0;i<32;i++) a[i]=0.0;
  }
  double operator()(int n) {
    if (n<=1) return 0.0;
    if (n<=31) return (a[n]>0.0) ? a[n] : (a[n]=lgamma((double)(n+1)));	
    else return lgamma((double)(n+1));
  };
 private:
  double a[32];
};

class success_rate {
 public:
  success_rate() {
    success=trials=0.0;
  }
  void operator+=(bool att) {
    if (att) success+=1.0;
    trials+=1.0;
  }
  void operator++() {
    success+=1.0;
    trials +=1.0;
  }
  void operator--() {
    trials+=1.0;
  }
  double operator()() {
    if (fabs(trials)<0.0001) return 0.0;   
    return success/trials;
  }
  void clear() {
    success=0.0;
    trials=0.0;
  }
 private:
  double success;
  double trials;
};


/** A template function that removes all but the last copy of
    a value in a vector - there must be a better way as this is 
    order I am not happy with this as the original iterator based one 
    was better but failed when iterators fell off the end
*/
 
template <typename T>
void keep_first_copy(std::vector<T> &a)
{
  // first remove adjacent values - cause problems for the rest
  // of the algorithma and this is more efficient
  a.erase(std::unique(a.begin(),a.end()),a.end());
  std::set<T> s;
  typename std::vector<T>::iterator i=a.begin();
  while (i!=a.end()) {
    if (s.count(*i)==0) {
	  s.insert(*i);
	  i++;
    } else { 
      i=a.erase(i);
    }
  }      
}


#endif
